package convert

import (
	"errors"
	"fmt"
	"git.chillcy.com/golang/chillcy/pkg/sdk/pc3"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mongo"
	productModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/product"
	sitePlatformPriceConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/site_platform_price_config"
	"github.com/zeromicro/go-zero/core/logx"
	"log"
	"math"
	"time"
)

type PlatformStoreSku struct {
	mongoManager *mongo.Manager
}

func (receiver *PlatformStoreSku) Convert(p productModel.Product) productModel.ProductColorSkuPlatformStoreSkus {
	var platformStoreSkus []productModel.ProductColorSkuPlatformStoreSkus

	for _, color := range p.Colors {
		for _, sku := range color.Skus {
			sitePlatformPriceConfigs := receiver.mongoManager.SitePlatformPriceConfigModel.GetCacheMany(func(document sitePlatformPriceConfigproductModel.SitePlatformPriceConfig) bool {
				return document.Site == sku.SiteId
			})
			categoryIds := p.Category.GetIds()
			for _, sitePlatformPriceConfig := range sitePlatformPriceConfigs {
				platformPriceDetail, err := r.calculatePlatformPrice(ctx, localSkuModel, pc3SkuModel, categories, priceConfig, svcProductModel)
				if err != nil {
					return nil, err
				}
				if platformPriceDetail != nil {
					skuPriceList = append(skuPriceList, *platformPriceDetail)
				}
			}
		}
	}
	err = r.dataRepo.BatchUpsertSkuPrice(ctx, skuPriceList)
	if err != nil {
		return nil, fmt.Errorf("BatchUpsertSkuPrice:%w", err)
	}
}

func (receiver *PlatformStoreSku) calculatePlatformPrice(localSkuModel *productModel.Sku, pc3SkuModel pc3.ModelSku, categories []string, priceConfig sitePlatformPriceConfigModel.SitePlatformPriceConfig, svcProductModel *ProductModel) (productModel.ProductColorSkuPlatformStoreSku, error) {
	var sku productModel.ProductColorSkuPlatformStoreSku
	if !priceConfig.IsValid() {
		return sku, fmt.Errorf("路线价格配置不合法")
	}
	sku, err := r.calculateRoutePrice(ctx, svcProductModel, localSkuModel, pc3SkuModel, categories, priceConfig.RoutePriceConfig, priceConfig.Seller)
	if err != nil {
		return sku, err
	}
	return sku, nil
}

func (receiver *PlatformStoreSku) calculateRoutePrice(svcProductModel *ProductModel, localSkuModel *model.Sku, pc3SkuModel pc3.ModelSku, categories []string, routeConfig embed.RoutePriceConfig, seller embed.Seller) (*model.SkuPrice, error) {

	adapter, err := priceRouteAdapter.SelectAdapter(routeConfig.Route)
	if err != nil {
		return nil, err
	}

	rateKey := pc3SkuModel.PriceDetail.Currency + "_" + string(routeConfig.Currency)
	currencyRate, err := r.rateConfig.GetRate(rateKey, seller.Platform)
	if err != nil || currencyRate == 0 {
		currencyRate, err = r.updateOrSetRate(rateKey, seller.Platform)
		if err != nil {
			return nil, errors.New("无法获取汇率信息")
		}
	}

	oldSkuPriceRecord, err := r.dataRepo.GetOneSkuPrice(ctx, localSkuModel.Id, seller.Platform, seller.Store, routeConfig.Route)
	if err != nil && !errors.Is(err, mongo.ErrNoDocuments) {
		return nil, err
	}

	subjectRate, subjectId, err := r.getSubjectRateAndSubject(pc3SkuModel, svcProductModel, seller.Platform)
	if err != nil {
		return nil, err
	}
	// 固定毛利
	margin := 1.0
	if oldSkuPriceRecord != nil && oldSkuPriceRecord.FixMargin != nil && *oldSkuPriceRecord.FixMargin != 0 {
		margin = 1 + (*oldSkuPriceRecord.FixMargin / 100)
	} else {
		margin, err = r.calculateMargin(routeConfig, svcProductModel, pc3SkuModel, seller)
		if err != nil {
			return nil, err
		}
	}
	margin = math2.Round(math.Max(1.0, margin), 3) // 保底1.0
	freight, err := r.calculateFreight(pc3SkuModel, categories, routeConfig.FreightConfig, seller.Platform)
	if err != nil {
		return nil, err
	}
	priceDetail := embed.RoutePriceDetail{
		Route:          routeConfig.Route,
		Currency:       string(routeConfig.Currency),
		OriginCurrency: pc3SkuModel.PriceDetail.Currency,
		Rate:           currencyRate,
		Margin:         margin,
		SubjectRate:    subjectRate,
		Subject:        subjectId,
		DiscountRate:   pc3SkuModel.PriceDetail.Discount,
		TotalDiscount:  pc3SkuModel.PriceDetail.Discount * subjectRate,
		TaxRate:        1 + routeConfig.TaxRate,
		Freight:        freight,
		Express:        r.calculateExpress(categories, routeConfig.ExpressConfig),
	}

	// 计算最终价格
	priceDetail.Price, err = adapter.GetPrice(pc3SkuModel.PriceDetail.Now, priceDetail, platform.Platform(seller.Platform))
	if err != nil {
		return nil, err
	}

	rateKey = priceDetail.Currency + "_" + string(embed.CurrencyCNY)
	cnyRate, err := r.rateConfig.GetRate(rateKey, seller.Platform)
	if err != nil || cnyRate == 0 {
		cnyRate, err = r.updateOrSetRate(rateKey, seller.Platform)
		if err != nil {
			log.Println(rateKey+"无法获取汇率信息:", err)
		}
	}

	record := model.SkuPrice{
		SkuId:          localSkuModel.Id,
		Mid:            svcProductModel.Id,
		Platform:       seller.Platform,
		Store:          seller.Store,
		Qty:            localSkuModel.Quantity,
		Route:          priceDetail.Route,
		Currency:       priceDetail.Currency,
		OriginCurrency: priceDetail.OriginCurrency,
		Price:          priceDetail.Price,
		Rate:           priceDetail.Rate,
		CnyRate:        cnyRate,
		Margin:         priceDetail.Margin,
		SubjectRate:    priceDetail.SubjectRate,
		Subject:        priceDetail.Subject,
		DiscountRate:   priceDetail.DiscountRate,
		TotalDiscount:  priceDetail.TotalDiscount,
		TaxRate:        priceDetail.TaxRate,
		Freight:        priceDetail.Freight,
		Express:        priceDetail.Express,
	}

	if oldSkuPriceRecord != nil {
		now := time.Now()
		change := math2.Round((priceDetail.Price-oldSkuPriceRecord.Price)/oldSkuPriceRecord.Price, 4)
		record.Id = oldSkuPriceRecord.Id
		record.FixedQty = oldSkuPriceRecord.FixedQty
		if math.Abs(change) > 0.0001 {
			record.ChangeRate = &change
			record.ChangeAt = &now
		}
	}
	return &record, nil
}
